@@ -29,6 +29,7 @@
  */
 
 #include "../include/config.h"
+#include "../include/common.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -41,12 +42,13 @@
 #include <string.h>
 #include <ctype.h>
 #include <netdb.h>
-#include <syslog.h>
 #include <stdarg.h>
 
 #include "../include/acl.h"
 
-/* This function checks if a char argumnet from valid char range.
+extern int debug;
+
+/* This function checks if a char argument from valid char range.
  * Valid range is: ASCII only, a number or a letter, a space, a dot, a slash, a dash, a comma.
  *
  * Returns:
@@ -76,16 +78,12 @@ int isvalidchar(int c) {
         switch (c) {
         case '.':
                 return 4;
-                break;
         case '/':
                 return 5;
-                break;
         case '-':
                 return 6;
-                break;
         case ',':
                 return 7;
-                break;
         default:
                 return 0;
         }
@@ -142,18 +140,27 @@ int add_ipv4_to_acl(char *ipv4) {
         unsigned long ip, mask;
         struct ip_acl *ip_acl_curr;
 
+		if(debug == TRUE)
+			logit(LOG_INFO, "add_ipv4_to_acl: checking ip-address >%s<", ipv4);
+
         /* Check for min and max IPv4 valid length */
-        if (len < 7 || len > 18)
-                return 0;
+		if (len < 7 || len > 18) {
+			logit(LOG_INFO, "add_ipv4_to_acl: Error, ip-address >%s< incorrect length", ipv4);
+			return 0;
+		}
 
         /* default mask for ipv4 */
         data[4] = 32;
 
         /* Basic IPv4 format check */
         for (i = 0; i < len; i++) {
-                /* Return 0 on error state */
-                if (state == -1)
-                        return 0;
+			/* Return 0 on error state */
+			if (state == -1) {
+				if(debug == TRUE)
+					logit(LOG_INFO, "add_ipv4_to_acl: Error, ip-address >%s< incorrect "
+								"format, continue with next check ...", ipv4);
+				return 0;
+			}
 
                 c = ipv4[i];
 
@@ -201,6 +208,7 @@ int add_ipv4_to_acl(char *ipv4) {
                 break;
         default:
                 /* Bad states */
+                logit(LOG_INFO, "add_ipv4_to_acl: Error, ip-address >%s< bad state", ipv4);
                 return 0;
         }
 
@@ -209,29 +217,29 @@ int add_ipv4_to_acl(char *ipv4) {
          */
         for (i=0; i < 4; i++) {
                 if (data[i] < 0 || data[i] > 255) {
-                        syslog(LOG_ERR,"Invalid IPv4 address/network format(%s) in allowed_hosts option\n",ipv4);
+                        logit(LOG_ERR,"Invalid IPv4 address/network format(%s) in allowed_hosts option\n",ipv4);
                         return 0;
                 }
         }
 
         if (data[4] < 0 || data[4] > 32) {
-                syslog(LOG_ERR,"Invalid IPv4 network mask format(%s) in allowed_hosts option\n",ipv4);
+                logit(LOG_ERR,"Invalid IPv4 network mask format(%s) in allowed_hosts option\n",ipv4);
                 return 0;
         }
 
-        /* Conver ip and mask to unsigned long */
+        /* Convert ip and mask to unsigned long */
         ip = htonl((data[0] << 24) + (data[1] << 16) + (data[2] << 8) + data[3]);
         mask =  htonl(-1 << (32 - data[4]));
 
         /* Wrong network address */
         if ( (ip & mask) != ip) {
-                syslog(LOG_ERR,"IP address and mask do not match in %s\n",ipv4);
+                logit(LOG_ERR,"IP address and mask do not match in %s\n",ipv4);
                 return 0;
         }
 
         /* Add addr to ip_acl list */
         if ( (ip_acl_curr = malloc(sizeof(*ip_acl_curr))) == NULL) {
-                syslog(LOG_ERR,"Can't allocate memory for ACL, malloc error\n");
+                logit(LOG_ERR,"Can't allocate memory for ACL, malloc error\n");
                 return 0;
         }
 
@@ -247,6 +255,10 @@ int add_ipv4_to_acl(char *ipv4) {
                 ip_acl_prev->next = ip_acl_curr;
         }
         ip_acl_prev = ip_acl_curr;
+
+        if(debug == TRUE)
+          logit(LOG_INFO, "add_ipv4_to_acl: ip-address >%s< correct, adding.", ipv4);
+
         return 1;
 }
 
@@ -271,7 +283,7 @@ int add_ipv6_to_acl(char *ipv6) {
 		messages if needed */
 	ipv6tmp = strdup(ipv6);
 	if(NULL == ipv6tmp) {
-		syslog(LOG_ERR, "Memory allocation failed for copy of address: %s\n", 
+		logit(LOG_ERR, "Memory allocation failed for copy of address: %s\n", 
 				ipv6);
 		return 0;
 		}
@@ -327,7 +339,7 @@ int add_ipv6_to_acl(char *ipv6) {
 	/* Add address to ip_acl list */
 	ip_acl_curr = malloc(sizeof(*ip_acl_curr));
 	if(NULL == ip_acl_curr) {
-		syslog(LOG_ERR, "Memory allocation failed for ACL: %s\n", ipv6);
+		logit(LOG_ERR, "Memory allocation failed for ACL: %s\n", ipv6);
 		return 0;
 		}
 
@@ -387,8 +399,12 @@ int add_domain_to_acl(char *domain) {
 
         struct dns_acl *dns_acl_curr;
 
-        if (len > 63)
+        if (len > 63) {
+                logit(LOG_INFO,
+					   "ADD_DOMAIN_TO_ACL: Error, did not add >%s< to acl list, too long!",
+					   domain);
                 return 0;
+        }
 
         for (i = 0; i < len; i++) {
                 c = domain[i];
@@ -426,7 +442,10 @@ int add_domain_to_acl(char *domain) {
                         }
                         break;
                 default:
-                        /* Not valid chars */
+                        logit(LOG_INFO,
+							   "ADD_DOMAIN_TO_ACL: Error, did not add >%s< to acl list, "
+								"invalid chars!", domain);
+					/* Not valid chars */
                         return 0;
                 }
         }
@@ -436,7 +455,7 @@ int add_domain_to_acl(char *domain) {
         case 1: case 4: case 5:
                 /* Add name to domain ACL list */
                 if ( (dns_acl_curr = malloc(sizeof(*dns_acl_curr))) == NULL) {
-                        syslog(LOG_ERR,"Can't allocate memory for ACL, malloc error\n");
+                        logit(LOG_ERR,"Can't allocate memory for ACL, malloc error\n");
                         return 0;
                 }
                 strcpy(dns_acl_curr->domain, domain);
@@ -448,13 +467,18 @@ int add_domain_to_acl(char *domain) {
                         dns_acl_prev->next = dns_acl_curr;
 
                 dns_acl_prev = dns_acl_curr;
+                if(debug == TRUE)
+                     logit(LOG_INFO, "ADD_DOMAIN_TO_ACL: added >%s< to acl list!", domain);
                 return 1;
         default:
+                logit(LOG_INFO,
+					   "ADD_DOMAIN_TO_ACL: ERROR, did not add >%s< to acl list, "
+						"check allowed_host in config file!", domain);
                 return 0;
         }
 }
 
-/* Checks connectiong host in ACL
+/* Checks connection host in ACL
  *
  * Returns:
  * 1 - on success
@@ -470,14 +494,23 @@ int is_an_allowed_host(int family, void *host)
 	struct sockaddr_in	*addr;
 	struct sockaddr_in6	addr6;
 	struct addrinfo		*res, *ai;
+	struct in_addr		tmp;
 
 	while (ip_acl_curr != NULL) {
 		if(ip_acl_curr->family == family) {
 			switch(ip_acl_curr->family) {
 			case AF_INET:
+				if (debug == TRUE) {
+					tmp.s_addr = ((struct in_addr*)host)->s_addr;
+					logit(LOG_INFO, "is_an_allowed_host (AF_INET): is host >%s< "
+							"an allowed host >%s<\n",
+						 inet_ntoa(tmp), inet_ntoa(ip_acl_curr->addr));
+				}
 				if((((struct in_addr *)host)->s_addr & 
 						ip_acl_curr->mask.s_addr) == 
 						ip_acl_curr->addr.s_addr) {
+					if (debug == TRUE)
+						logit(LOG_INFO, "is_an_allowed_host (AF_INET): host is in allowed host list!");
 					return 1;
 					}
 				break;
@@ -509,9 +542,20 @@ int is_an_allowed_host(int family, void *host)
 				switch(ai->ai_family) {
 
 				case AF_INET:
+					if(debug == TRUE) {
+						tmp.s_addr=((struct in_addr *)host)->s_addr;
+						logit(LOG_INFO, "is_an_allowed_host (AF_INET): is host >%s< "
+								"an allowed host >%s<\n",
+							 inet_ntoa(tmp), dns_acl_curr->domain);
+					}
+
 					addr = (struct sockaddr_in*)(ai->ai_addr);
-					if (addr->sin_addr.s_addr == ((struct in_addr*)host)->s_addr)
+					if (addr->sin_addr.s_addr == ((struct in_addr*)host)->s_addr) {
+						if (debug == TRUE)
+							logit(LOG_INFO, "is_an_allowed_host (AF_INET): "
+									"host is in allowed host list!");
 						return 1;
+					}
 					break;
 
 				case AF_INET6:
@@ -521,9 +565,9 @@ int is_an_allowed_host(int family, void *host)
 					break;
 				}
 			}
-
-			dns_acl_curr = dns_acl_curr->next;
 		}
+
+		dns_acl_curr = dns_acl_curr->next;
 	}
 	return 0;
 }
@@ -559,19 +603,30 @@ void parse_allowed_hosts(char *allowed_hosts) {
 	const char *delim = ",";
 	char *trimmed_tok;
 
+	if (debug == TRUE)
+		logit(LOG_INFO,
+			 "parse_allowed_hosts: parsing the allowed host string >%s< to add to ACL list\n",
+			 allowed_hosts);
+
 #ifdef HAVE_STRTOK_R
 	tok = strtok_r(hosts, delim, &saveptr);
 #else
+	if (debug == TRUE)
+		logit(LOG_INFO,"parse_allowed_hosts: using strtok, this might lead to "
+				"problems in the allowed_hosts string determination!\n");
 	tok = strtok(hosts, delim);
 #endif
 	while( tok) {
 		trimmed_tok = malloc( sizeof( char) * ( strlen( tok) + 1));
 		trim( tok, trimmed_tok);
+		if(debug == TRUE)
+			logit(LOG_DEBUG, "parse_allowed_hosts: ADDING this record (%s) to ACL list!\n", trimmed_tok);
 		if( strlen( trimmed_tok) > 0) {
 			if (!add_ipv4_to_acl(trimmed_tok) && !add_ipv6_to_acl(trimmed_tok) 
 					&& !add_domain_to_acl(trimmed_tok)) {
-				syslog(LOG_ERR,"Can't add to ACL this record (%s). Check allowed_hosts option!\n",trimmed_tok);
-			}
+				logit(LOG_ERR,"Can't add to ACL this record (%s). Check allowed_hosts option!\n",trimmed_tok);
+			} else if (debug == TRUE)
+				logit(LOG_DEBUG,"parse_allowed_hosts: Record added to ACL list!\n");
 		}
 		free( trimmed_tok);
 #ifdef HAVE_STRTOK_R
@@ -606,17 +661,21 @@ unsigned int prefix_from_mask(struct in_addr mask) {
  * It shows all hosts in ACL lists
  */
 
-void show_acl_lists(void) {
-        struct ip_acl *ip_acl_curr = ip_acl_head;
-        struct dns_acl *dns_acl_curr = dns_acl_head;
+void show_acl_lists(void)
+{
+	struct ip_acl *ip_acl_curr = ip_acl_head;
+	struct dns_acl *dns_acl_curr = dns_acl_head;
 
-        while (ip_acl_curr != NULL) {
-                printf(" IP ACL: %s/%u %u\n", inet_ntoa(ip_acl_curr->addr), prefix_from_mask(ip_acl_curr->mask), ip_acl_curr->addr.s_addr);
-                ip_acl_curr = ip_acl_curr->next;
-        }
+	logit(LOG_INFO, "Showing ACL lists for both IP and DOMAIN acl's:\n" );
 
-        while (dns_acl_curr != NULL) {
-                printf("DNS ACL: %s\n", dns_acl_curr->domain);
-                dns_acl_curr = dns_acl_curr->next;
-        }
+	while (ip_acl_curr != NULL) {
+		logit(LOG_INFO, "   IP ACL: %s/%u %u\n", inet_ntoa(ip_acl_curr->addr),
+			 prefix_from_mask(ip_acl_curr->mask), ip_acl_curr->addr.s_addr);
+		ip_acl_curr = ip_acl_curr->next;
+	}
+
+	while (dns_acl_curr != NULL) {
+		logit(LOG_INFO, "  DNS ACL: %s\n", dns_acl_curr->domain);
+		dns_acl_curr = dns_acl_curr->next;
+	}
 }
